def get_schema():
    schema = {
        "description": "publish data via email",
        "type": "object",
        "properties": {
            "address": {
                "description": "url",
                "type": "string"
            },
            "port": {
                "description": "port (e.g.: 587)",
                "type": "integer",
                "minimum": 0,
                "maximum": 65535
            },
            "credentials-file": {
                "description": "File containing the credentials (optional).",
                "type": "string"
            },
            "username": {
                "description": "username for smtp server",
                "type": "string"
            },
            "password": {
                "description": "password for smtp server",
                "type": "string"
            },
            "from": {
                "description": "sender e-mail address",
                "type": "string"
            },
            "to": {
                "description": "receiver e-mail address",
                "oneOf": [
                    {"type": "string"},
                    {
                        "type": "array",
                        "items": {
                            "type": "string"
                        }
                    }
                ]
            }
        },
        "additionalItems": False,
        "required": ["address", "port", "from", "to"]
    }

    return schema



from lysidike.services.aservice import AService
import datetime


class StdOutService(AService):
    """
    Writes the collected messages to stdout.

    stdout:
            prefix: ------------  # first line of output
            suffix: ............  # last line of output
    """
    _prefix = None
    _suffix = None

    def __init__(self, config, mqtt_client, logger):
        """
        Constructor

        :param config: yaml config structure
        :param mqtt_client: mqtt_client instance
        :param logger: logger instance from the parent. a child will be spawned
        """
        AService.__init__(self, config, mqtt_client, __name__, logger)
        self._prefix = self._config["prefix"]
        self._suffix = self._config["suffix"]

    def _start(self):
        pass

    def _stop(self):
        pass

    def _test_connection(self):
        return True

    def _render_message(self, messages):
        body = "received {} messages since last update.\n".format(len(messages))
        body += "\nMessages:\n"
        for entry in messages:
            timestamp, topic, message = entry
            timestamp = timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
            body += "  {}; {}; '{}'\n".format(timestamp, topic, message)
        body += "EOF\n"
        return body

    def _send_message(self, subject, message):
        self._logger.info("Writing to stdout")
        print(self._prefix)
        print("@{}".format(datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ")))
        print(subject)
        print()
        print(message)
        print(self._suffix)



def get_schema():
    schema = {
        "description": "publish to stdout",
        "type": "object",
        "properties": {
            "prefix": {
                "description": "prefix for stdout",
                "type": "string"
            },
            "suffix": {
                "description": "suffix for stdout",
                "type": "string"
            }
        },
        "additionalItems": False,
        "required": ["prefix", "suffix"]
    }

    return schema



from lysidike.services.aservice import AService
import smtplib
from email.mime.text import MIMEText


class EmailService(AService):
    """
    Classic publish to email service.

    config yaml entries:

    email:
            address: smtp.zoho.com  # url of smtp server
            port: 465  # port of smtp server
            from: name123@sender123.com  # sender email address
            to: name123@receiver123.com  # send to email address. either single string or array of strings
            credentials-file: ~/credentials.yaml  # a yaml config file with one root entry as described below

    credentials yaml file:

    publish-gateway:
        services:
            email:
                username: username  # username for smtp server
                password: password  # password for smtp server
    """

    _address = None  # url of smtp server
    _port = None  # port of smtp server
    _username = None  # username for smtp server
    _password = None  # password for smtp server
    _from = None  # sender email address
    _to = None  # send to email address

    def __init__(self, config, mqtt_client, logger):
        """
        Constructor

        :param config: yaml config structure
        :param mqtt_client: mqtt_client instance
        :param logger: logger instance from the parent. a child will be spawned
        """

        AService.__init__(self, config, mqtt_client, __name__, logger)
        self._address = self._config["address"]
        self._port = self._config["port"]
        self._username = self._config["username"]
        self._password = self._config["password"]
        self._from = self._config["from"]
        if isinstance(self._config["to"], str):
            self._to = [self._config["to"]]
        else:
            self._to = self._config["to"]

    def _start(self):
        if not self._test_connection():
            self._logger.error("EmailService._start - connection test failed")
            raise RuntimeError("EmailService._start - connection test failed")

    def _stop(self):
        pass

    def _test_connection(self):
        try:
            server = smtplib.SMTP_SSL(host=self._address, port=self._port)
            server.set_debuglevel(0)
            server.login(self._username, self._password)
            server.noop()
            server.quit()
            return True
        except smtplib.SMTPException as exception:
            self._logger.error("EmailService._test_connection - failed: {}.".format(exception))
        return False

    def _render_message(self, messages):
        body = "received {} messages since last update.\n".format(len(messages))
        body += "\nMessages:\n"
        for entry in messages:
            timestamp, topic, message = entry
            timestamp = timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
            body += "  {}; {}; '{}'\n".format(timestamp, topic, message)
        body += "EOF\n"
        return body

    def _send_message(self, subject, message):
        self._logger.debug("EmailService._send_message")
        msg = MIMEText(message)
        msg['To'] = ', '.join(self._to)
        msg['From'] = self._from
        msg['Subject'] = subject

        try:
            server = smtplib.SMTP_SSL(host=self._address, port=self._port)
            server.set_debuglevel(0)
            server.login(self._username, self._password)
            server.send_message(msg)
        except smtplib.SMTPException as exception:
            self._logger.error("EmailService._send_message - failed: {}.".format(exception))
            raise
        finally:
            server.quit()



import unittest
import os
import io
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from pelops.myconfigtools import read_config, validate_config
from pelops.mylogger import create_logger
from pelops.mymqttclient import MyMQTTClient
from lysidike.services.emailservice import EmailService
import datetime
from contextlib import redirect_stdout


class TestEMAILService(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')) +
                                 "/tests_unit/config.yaml")
        cls.logger = create_logger(cls.config["logger"], "TestEMAILService")
        cls.logger.info("start")
        cls.mqtt_client = MyMQTTClient(cls.config["mqtt"], cls.logger)
        cls.mqtt_client.connect()
        cls.service_config = cls.config["publish-gateway"]["services"]["email"]
        # assume that username is a valid email address for the account - i dont want to place a real
        # email address into the config yaml file ...
        cls.service_config["from"] = cls.service_config["username"]
        cls.service_config["to"] = cls.service_config["username"]

    @classmethod
    def tearDownClass(cls):
        cls.mqtt_client.disconnect()
        cls.logger.info("end")

    def setUp(self):
        self.logger.info("-----------------------------------------------------------------------")

    def tearDown(self):
        self.logger.info(".......................................................................")

    def test_0init(self):
        stdout = EmailService(self.service_config, self.mqtt_client, self.logger)
        self.assertIsNotNone(stdout)

    def test_1startstop(self):
        stdout = EmailService(self.service_config, self.mqtt_client, self.logger)
        self.assertIsNotNone(stdout)
        stdout.start()
        stdout.stop()

    def test_2testconnection(self):
        stdout = EmailService(self.service_config, self.mqtt_client, self.logger)
        self.assertIsNotNone(stdout)
        stdout.start()
        self.assertTrue(stdout._test_connection())
        stdout.stop()

    def test_3rendermessages(self):
        stdout = EmailService(self.service_config, self.mqtt_client, self.logger)
        self.assertIsNotNone(stdout)
        messages = []
        start = datetime.date(year=2018, month=1, day=1)
        for i in range(2):
            timestamp = start + datetime.timedelta(days=i)
            topic = "\\test\\test"
            m = i
            messages.append((timestamp, topic, m))
        message = stdout._render_message(messages)

        expected_message = "received 2 messages since last update.\n\n"
        expected_message += "Messages:\n"
        expected_message += "  2018-01-01T00:00:00.000000Z; \\test\\test; '0'\n"
        expected_message += "  2018-01-02T00:00:00.000000Z; \\test\\test; '1'\n"
        expected_message += "EOF\n"

        self.assertEqual(message, expected_message)

    def test_4sendmessage(self):
        email = EmailService(self.service_config, self.mqtt_client, self.logger)
        self.assertIsNotNone(email)
        message = "test message @{}.".format(datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ"))
        subject = "test subject"
        email._send_message(subject, message)

    def test_5sendmessage_multiple_tos(self):
        self.service_config["to"] = [self.service_config["to"], self.service_config["to"]]
        email = EmailService(self.service_config, self.mqtt_client, self.logger)
        self.assertIsNotNone(email)
        message = "test message @{}.".format(datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ"))
        subject = "test subject"
        email._send_message(subject, message)

if __name__ == '__main__':
    unittest.main()



from pelops import mylogger
from lysidike.tasks.task import Task


class TaskFactory:
    @staticmethod
    def get_tasks(config, services, mqtt_client, logger):
        """
        Creates a list of tasks.

        :param config: config yaml structure
        :param services: list of services created by service factory
        :param mqtt_client: mqtt client instance
        :param logger: logger instance
        :return: list of tasks
        """
        factory_logger = mylogger.get_child(logger, __name__)
        factory_logger.info("creating tasks - starting")
        factory_logger.debug("task configs: ".format(config))
        factory_logger.debug("task configs: ".format(config))

        tasks = {}
        for entry in config:
            task = Task(entry, services, mqtt_client, logger)
            tasks[task.name] = task
            factory_logger.info("added task {}".format(task.name))

        factory_logger.info("creating tasks - finished")
        return tasks



import lysidike.schema.emailservice
import lysidike.schema.stdoutservice


def get_schema():
    schema = {
        "description": "list of services for publishing",
        "type": "object",
        "properties": {
            "email": lysidike.schema.emailservice.get_schema(),
            "stdout": lysidike.schema.stdoutservice.get_schema()
        },
        "additionalItems": False
    }
    return schema


from pelops.abstractmicroservice import AbstractMicroservice
from lysidike.services.servicefactory import ServiceFactory
from lysidike.tasks.taskfactory import TaskFactory
from lysidike.schema.publishgateway import get_schema
import lysidike


class PublishGateway(AbstractMicroservice):
    """
    Provides the possibility to publish aggregated mqtt messages to various services. Several tasks listen to
    incoming messages, render messages and publishes them to the service.

    config yaml structure
    mqtt:
        mqtt-address: localhost
        mqtt-port: 1883
        credentials-file: ~/credentials.yaml
        log-level: WARNING

    logger:
        log-level: DEBUG
        log-file: lysidike.log

    publish-gateway:
        services:
            email:
                address: smtp.zoho.com
                port: 465
                from: name123@sender123.com
                to: name123@receiver123.com
                credentials-file: ~/credentials.yaml
            stdout:
                prefix: ------------------------------------------------------------------------------
                suffix: ..............................................................................
        tasks:
          - name: collection
            service: stdout  # [email, stdout, ...] one of the services
            subject: sending collected messages  # subject for message
            every-nth-message: 10  # collect this amount of messages before publishing them. 0 for never
            every-nth-second: 10  # wait this time before publishing all messages that are waiting. 0 for never
            topics-sub:
                - /test/a
                - /test/b
          - name: ...

    """
    _version = lysidike.version

    _services = None  # list of services
    _tasks = None  # list of tasks

    def __init__(self, config, mqtt_client=None, logger=None):
        """
        Constructor - creates the services and the tasks

        :param config: config yaml structure
        :param mqtt_client: mqtt client instance
        :param logger: logger instance
        """
        AbstractMicroservice.__init__(self, config, "publish-gateway", mqtt_client, logger)
        self._services = ServiceFactory.get_services(self._config["services"], mqtt_client, logger)
        self._tasks = TaskFactory.get_tasks(self._config["tasks"], self._services, mqtt_client, logger)

    def _start(self):
        self._logger.info("PublishGateway._start - starting services")
        for service in self._services.values():
            service.start()
        self._logger.info("PublishGateway._start - starting tasks")
        for task in self._tasks.values():
            task.start()
        self._logger.info("PublishGateway._start - finished")

    def _stop(self):
        self._logger.info("PublishGateway._stop - stopping tasks")
        for task in self._tasks.values():
            task.stop()
        self._logger.info("PublishGateway._stop - stopping services")
        for service in self._services.values():
            service.stop()
        self._logger.info("PublishGateway._stop - finished")

    @classmethod
    def _get_description(cls):
        return "Lysidike publishes incoming mqtt messages to various internet services like email."

    @classmethod
    def _get_schema(cls):
        return get_schema()


def standalone():
    PublishGateway.standalone()


if __name__ == "__main__":
    PublishGateway.standalone()



from pelops import mylogger


class ServiceFactory:
    """
    generate instances of the services
    """

    @staticmethod
    def get_service(service_type, config, mqtt_client, logger):
        """
        Create a service with the provided parameters

        :param service_type: name of service type
        :param config: config parameters for service
        :param mqtt_client: mqtt client instance
        :param logger: logger instance
        :return: instance of the service
        """
        factory_logger = mylogger.get_child(logger, __name__)
        factory_logger.info(" - creating service")
        factory_logger.debug(" - service config: ".format(config))

        if service_type == "email":
            from lysidike.services.emailservice import EmailService
            service = EmailService(config, mqtt_client, logger)
        elif service_type == "stdout":
            from lysidike.services.stdoutservice import StdOutService
            service = StdOutService(config, mqtt_client, logger)
        else:
            factory_logger.error("ServiceFactory.get_service - unknown type '{}'.".format(service_type))
            raise ValueError("ServiceFactory.get_service - unknown type '{}'.".format(service_type))

        return service

    @staticmethod
    def get_services(config, mqtt_client, logger):
        """
        creates a list of serives.

        config yaml structures:
        email:
            address: smtp.zoho.com
            port: 465
            from: name123@sender123.com
            to: name123@receiver123.com
            credentials-file: ~/credentials.yaml
        stdout:
            prefix: ------------------------------------------------------------------------------
            suffix: ..............................................................................

        :param config: config yaml structure
        :param mqtt_client: mqtt client instance
        :param logger: logger instance
        :return: list of service instances
        """
        factory_logger = mylogger.get_child(logger, __name__)
        factory_logger.info("creating services - starting")
        factory_logger.debug("service configs: ".format(config))
        services = {}
        for service_type, service_config in config.items():
            service_type = service_type.lower()
            services[service_type] = ServiceFactory.get_service(service_type, service_config, mqtt_client, logger)
            factory_logger.info("added service {}".format(service_type))

        factory_logger.info("creating services - finished")

        return services



from pelops import mylogger
import threading
import datetime


class Task:
    """
    A task collects messages from n topics and publishes them a specific service after x seconds and/or y messages.
    Each call of publish the list of received messages is processed and emptied. Thus, two consecutive calls of
    publish will produce two different messages (or two empties with an empty list). To ensure this behavior, a lock
    is used to block access to the list.

    config yaml structure:
      - name: collection
        service: stdout  # [email, stdout, ...] one of the services
        subject: sending collected messages  # subject for message
        every-nth-message: 10  # collect this amount of messages before publishing them. 0 for never
        every-nth-second: 10  # wait this time before publishing all messages that are waiting. 0 for never
        topics-sub:
            - /test/a
            - /test/b
            - /test/c
    """

    _config = None  # config yaml structure
    _mqtt_client = None  # mqtt client instance
    _logger = None  # logger instance

    _topics_sub = None  # list of subscribed topics
    _message_handler = None  # list of the handlers for the subscribed topics

    _service = None  # instance of the service to be used
    _subject = None  # subject for each publish message
    name = None  # name of the service

    _every_nth_message = None  # collect this amount of messages before publishing them. 0 for never
    _use_every_message = None  # true if _every_nth_message > 0

    _every_nth_second = None  # wait this time before publishing all messages that are waiting. 0 for never
    _use_every_second = None  # true if _use_every_second > 0
    _loop_thread = None  # thread for timer loop
    _stop_loop = None  # event signals that timer loop should stop

    _message_list = None  # list of unprocessed messages received from all topics
    _lock_list = None  # locks the message list (mutual exclusive access for renderer and handler)

    def __init__(self, config, services, mqtt_client, logger):
        """
        Constructor

        :param config: config yaml structure
        :param services: list of services created by servicefactory
        :param mqtt_client: mqtt client instances
        :param logger: logger instance
        """
        self._config = config
        self.name = self._config["name"]
        self._mqtt_client = mqtt_client
        self._logger = mylogger.get_child(logger, __name__ + "." + self.name)

        self._logger.info("Task.__init__ - start")
        self._logger.debug("Task.__init__ - config: {}".format(self._config))

        self._subject = self._config["subject"]
        self._topics_sub = self._config["topics-sub"]
        self._service = services[self._config["service"].lower()]

        self._every_nth_message = self._config["every-nth-message"]
        if self._every_nth_message == 0:
            self._use_every_message = False
        else:
            self._use_every_message = True

        self._every_nth_second = self._config["every-nth-second"]
        if self._every_nth_second == 0:
            self._use_every_second = False
        else:
            self._use_every_second = True
        self._loop_thread = threading.Thread(target=self._loop)
        self._stop_loop = threading.Event()
        self._stop_loop.set()

        self._message_handler = {}
        for topic in self._topics_sub:
            self._message_handler[topic] = self._create_handler(topic)

        self._message_list = []
        self._lock_list = threading.Lock()

        self._logger.info("Task.__init__ - finished")

    def _create_handler(self, handler_topic):
        """
        For each sub topic a handler is created that adds incoming messages to the list of messages that are processed
        into an outgoing message.
        """
        def _handler(message):
            timestamp = datetime.datetime.now()
            entry = (timestamp, handler_topic, message)
            self._logger.info("Task._handler.'{}' - received message".format(handler_topic))
            self._logger.debug("Task._handler.'{}' - {}".format(handler_topic, entry))
            with self._lock_list:
                self._logger.debug("Task._handler.'{}' - lock acquired".format(handler_topic))
                self._message_list.append(entry)
            self._logger.debug("Task._handler.'{}' - lock released".format(handler_topic))
            self._logger.debug("Task._handler.'{}' - added message to list".format(handler_topic))
            self._handler_post_processor()

        self._logger.debug("Task._create_handler - added handler {} for topic {}.".format(_handler, handler_topic))
        return _handler

    def _handler_post_processor(self):
        """
        after each added message to the list, the system checks if list size is larger than _every_nth_message. if
        yes, publish message is called.
        """
        if self._use_every_message and len(self._message_list) >= self._every_nth_message:
            self._logger.info("Task._handler_post_processer - message_list size: {}.".format(len(self._message_list)))
            self._publish()

    def _loop(self):
        """
        if _every_nth_second is set, this method waits n seconds and publishes all messages in the list.
        """
        self._logger.info("Task._loop - start loop")
        while not self._stop_loop.is_set():
            self._logger.info("Task._loop - wait for {} seconds.".format(self._every_nth_second))
            self._stop_loop.wait(self._every_nth_second)
            if self._stop_loop.is_set():
                continue
            self._logger.info("Task._loop - sending messages")
            self._publish()

    def _publish(self):
        """
        A copy of the received messages is given to the service handler and published. The stored messages are cleared
        afterwards. The list is protected by _lock_list during processing.
        """
        self._logger.info("Task._publish - start")
        with self._lock_list:
            self._logger.debug("Task._publish - lock acquired")
            messages = self._message_list.copy()
            self._logger.debug("Task._publish - copied {} messages.".format(len(messages)))
            self._message_list.clear()
        self._logger.debug("Task._publish - lock released")
        self._service.publish(self._subject, messages)
        self._logger.info("Task._publish - finished")

    def start(self):
        """
        Starts the task (if timer is used, a thread with _loop is started) and subscribes to all sub topics.
        """
        self._logger.info("Task.start - starting")
        if self._use_every_second:
            self._logger.info("Task.start - start loop")
            self._stop_loop.clear()
            self._loop_thread.start()
        self._logger.info("Task.start - register {} topics".format(len(self._message_handler.items())))
        for topic, handler in self._message_handler.items():
            self._logger.debug("Task.start - subscribing to topic {} the handler {}".format(topic, handler))
            self._mqtt_client.subscribe(topic, handler)
        self._logger.info("Task.start - finished")

    def stop(self):
        """
        Stops the task (and the timer thread) and unsubscribes from the topics.
        """
        self._logger.info("Task.stop - stopping")
        self._logger.info("Task.start - unregister {} topics".format(len(self._message_handler.items())))
        for topic, handler in self._message_handler.items():
            self._logger.debug("Task.start - unsubscribing from topic {} the handler {}".format(topic, handler))
            self._mqtt_client.unsubscribe(topic, handler)
        if self._use_every_second:
            self._logger.info("Task.start - stop loop")
            self._stop_loop.set()
            self._loop_thread.join()
        self._logger.info("Task.stop - finished")



from pelops import mylogger


class AService:
    """
    Abstract service - an implementation of it provides everything that is needed to publish the message
    to a messaging service like email or to output it to e.g. stdout.

    Abstact service has no config yaml entries.
    """

    _mqtt_client = None  # mqtt_client instance
    _logger = None  # logger
    _config = None  # yaml config structure
    last_message = None  # contains the last rendered message

    def __init__(self, config, mqtt_client, logger_name, logger):
        """
        Constructor

        :param config: yaml config structure
        :param mqtt_client: mqtt_client instance
        :param logger: logger instance from the parent. a child will be spawned
        :param logger_name: name for the child instance
        """
        self._logger = mylogger.get_child(logger, logger_name)
        self._mqtt_client = mqtt_client
        self._config = config

        self._logger.info("{}.__init__".format(self.__class__.__name__))
        self._logger.debug("{}.__init__ config: {}".format(self.__class__.__name__, self._config))

    def start(self):
        """
        start the service - calls self._start and
        :raises RuntimeError if self._test_connection returns fals
        """
        self._logger.info("{}.start - starting".format(self.__class__.__name__))
        self._start()
        if not self._test_connection():
            self._logger.error("{}.start - connection test failed".format(self.__class__.__name__))
            raise RuntimeError("{}.start - connection test failed".format(self.__class__.__name__))
        self._logger.info("{}.start - started".format(self.__class__.__name__))

    def stop(self):
        """
        stops the service - calls self._stop
        """
        self._logger.info("{}.stop - stopping".format(self.__class__.__name__))
        self._stop()
        self._logger.info("{}.stop - stopped".format(self.__class__.__name__))

    def publish(self, subject, messages):
        """
        Publish the topics to this service

        :param subject: string - subject for message
        :param messages: list of topics to be published. each entry is a tuple (datetime, topic, message)
        """
        self._logger.info("{}.publish - starting".format(self.__class__.__name__))
        self._logger.debug("{}.publish - subject: '{}', len messages: {}, messages: {}"
                           .format(self.__class__.__name__, subject, len(messages), messages))

        self.last_message = self._render_message(messages)
        self._logger.debug("{}.publish - message: ".format(self.__class__.__name__, self.last_message))

        self._send_message(subject, self.last_message)

        self._logger.info("{}.publish - finished".format(self.__class__.__name__))

    def _start(self):
        """
        initialise service
        """
        raise NotImplementedError

    def _stop(self):
        """
        stop service
        """
        raise NotImplementedError

    def _test_connection(self):
        """
        Test if the connection to the publishing service works - used during startup.
        :return: boolean. True if success
        """
        raise NotImplementedError

    def _render_message(self, messages):
        """
        takes the list of received messages (in the order they were received) and generates a message in a format
        that fits the service type

        :param messages: list of received messages. each entry is a tuple (timestamp, topic, message)
        :return: message
        """
        raise NotImplementedError

    def _send_message(self, subject, message):
        """
        sends a message with given subject to the service

        :param subject: subject as defined in config yaml - provided by task
        :param message: message to be published
        """
        raise NotImplementedError



import unittest
import os
import io
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from pelops.myconfigtools import read_config, validate_config
from pelops.mylogger import create_logger
from pelops.mymqttclient import MyMQTTClient
from lysidike.publishgateway import PublishGateway
import time


class TestPublishGateway(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')) +
                                 "/tests_unit/config.yaml")
        cls.logger = create_logger(cls.config["logger"], "TestPublishGateway")
        cls.logger.info("start")
        cls.mqtt_client = MyMQTTClient(cls.config["mqtt"], cls.logger)
        cls.mqtt_client.connect()

    @classmethod
    def tearDownClass(cls):
        cls.mqtt_client.disconnect()
        cls.logger.info("end")

    def setUp(self):
        self.logger.info("-----------------------------------------------------------------------")

    def tearDown(self):
        self.logger.info(".......................................................................")

    def test_00create(self):
        pg = PublishGateway(self.config, self.mqtt_client, self.logger)
        self.assertIsNotNone(pg)
        self.assertEqual(len(pg._services), len(self.config["publish-gateway"]["services"]))
        self.assertEqual(len(pg._tasks), len(self.config["publish-gateway"]["tasks"]))

    def test_00start_stop(self):
        pg = PublishGateway(self.config, self.mqtt_client, self.logger)
        self.assertIsNotNone(pg)
        pg.start()
        time.sleep(1)
        pg.stop()


if __name__ == '__main__':
    unittest.main()




import lysidike.schema.servicefactory
import lysidike.schema.task


def get_schema():
    schema = {
        "publish-gateway": {
            "description": "Lysidike publishes incoming mqtt messages to various internet services like email.",
            "type": "object",
            "properties": {
                "services": lysidike.schema.servicefactory.get_schema(),
                "tasks": {
                    "description": "all tasks",
                    "type": "array",
                    "items": lysidike.schema.task.get_schema(),
                    "additionalItems": False
                }
            },
            "required": ["services", "tasks"],
            "additionalItems": False
        }
    }

    return schema


import unittest
import os
import io
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from pelops.myconfigtools import read_config, validate_config
from pelops.mylogger import create_logger
from pelops.mymqttclient import MyMQTTClient
from lysidike.services.servicefactory import ServiceFactory
from lysidike.tasks.task import Task
from lysidike.services.stdoutservice import StdOutService
from lysidike.services.emailservice import EmailService
from contextlib import redirect_stdout
import time


class TestTask(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')) +
                                 "/tests_unit/config.yaml")
        cls.logger = create_logger(cls.config["logger"], "TestTask")
        cls.logger.info("start")

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        self.mqtt_client = MyMQTTClient(self.config["mqtt"], self.logger)
        self.mqtt_client.connect()

        self.services = ServiceFactory.get_services(self.config["publish-gateway"]["services"], self.mqtt_client,
                                                    self.logger)
        self.assertIsNotNone(self.services)
        self.logger.info("-----------------------------------------------------------------------")

    def tearDown(self):
        self.logger.info(".......................................................................")
        self.mqtt_client.disconnect()
        self.logger.info("end")

    def test_00init_stdout(self):
        config = {
            "name": "immediate",
            "service": "stdout",
            "subject": "immediate information",
            "every-nth-message": 1,
            "every-nth-second": 2,
            "topics-sub": ["/test/d", "/test/a"]
        }
        task = Task(config, self.services, self.mqtt_client, self.logger)
        self.assertIsNotNone(task)
        self.assertEqual(type(task._service), StdOutService)
        self.assertEqual("immediate", task.name)
        self.assertEqual("immediate information", task._subject)
        self.assertEqual(len(task._topics_sub), 2)
        self.assertEqual(task._topics_sub[0], "/test/d")
        self.assertEqual(task._topics_sub[1], "/test/a")
        self.assertEqual(task._every_nth_message, 1)
        self.assertEqual(task._every_nth_second, 2)

    def test_01init_email(self):
        config = {
            "name": "immediate",
            "service": "email",
            "subject": "immediate information",
            "every-nth-message": 1,
            "every-nth-second": 2,
            "topics-sub": ["/test/d", "/test/a"]
        }
        task = Task(config, self.services, self.mqtt_client, self.logger)
        self.assertIsNotNone(task)
        self.assertEqual(type(task._service), EmailService)
        self.assertEqual("immediate", task.name)
        self.assertEqual("immediate information", task._subject)
        self.assertEqual(len(task._topics_sub), 2)
        self.assertEqual(task._topics_sub[0], "/test/d")
        self.assertEqual(task._topics_sub[1], "/test/a")
        self.assertEqual(task._every_nth_message, 1)
        self.assertEqual(task._every_nth_second, 2)

    def test_02start_stop(self):
        config = {
            "name": "immediate",
            "service": "stdout",
            "subject": "immediate information",
            "every-nth-message": 1,
            "every-nth-second": 2,
            "topics-sub": ["/test/d", "/test/a"]
        }
        task = Task(config, self.services, self.mqtt_client, self.logger)
        self.assertIsNotNone(task)
        task.start()
        self.logger.info("... waiting 0.1 seconds ...")
        time.sleep(0.1)
        task.stop()

    def test_03immediate(self):
        config = {
            "name": "immediate",
            "service": "stdout",
            "subject": "immediate information",
            "every-nth-message": 1,
            "every-nth-second": 0,
            "topics-sub": ["/test/d"]
        }
        task = Task(config, self.services, self.mqtt_client, self.logger)
        self.assertIsNotNone(task)
        self.assertEqual(task._every_nth_message, 1)
        self.assertEqual(task._every_nth_second, 0)
        self.assertFalse(task._use_every_second)
        self.assertTrue(task._use_every_message)

        task.start()
        time.sleep(0.1)
        f = io.StringIO()
        with redirect_stdout(f):
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(0.1)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 11)
        f = io.StringIO()
        with redirect_stdout(f):
            self.mqtt_client.publish("/test/d", "a")
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(0.1)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 22)
        task.stop()

    def test_04collect(self):
        config = {
            "name": "immediate",
            "service": "stdout",
            "subject": "immediate information",
            "every-nth-message": 2,
            "every-nth-second": 0,
            "topics-sub": ["/test/d"]
        }
        task = Task(config, self.services, self.mqtt_client, self.logger)
        self.assertIsNotNone(task)
        self.assertEqual(task._every_nth_message, 2)
        self.assertEqual(task._every_nth_second, 0)
        self.assertFalse(task._use_every_second)
        self.assertTrue(task._use_every_message)

        task.start()
        time.sleep(0.1)
        f = io.StringIO()
        with redirect_stdout(f):
            self.mqtt_client.publish("/test/d", "a")
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(0.1)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 12)
        f = io.StringIO()
        with redirect_stdout(f):
            self.mqtt_client.publish("/test/d", "a")
            self.mqtt_client.publish("/test/d", "a")
            self.mqtt_client.publish("/test/d", "a")
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(0.1)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 24)
        task.stop()

    def test_05time(self):
        config = {
            "name": "immediate",
            "service": "stdout",
            "subject": "immediate information",
            "every-nth-message": 0,
            "every-nth-second": 1,
            "topics-sub": ["/test/d"]
        }
        task = Task(config, self.services, self.mqtt_client, self.logger)
        self.assertIsNotNone(task)
        self.assertEqual(task._every_nth_message, 0)
        self.assertEqual(task._every_nth_second, 1)
        self.assertTrue(task._use_every_second)
        self.assertFalse(task._use_every_message)

        task.start()
        time.sleep(0.1)
        f = io.StringIO()
        with redirect_stdout(f):
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(1)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 11)
        f = io.StringIO()
        with redirect_stdout(f):
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(0.1)
            self.mqtt_client.publish("/test/d", "a")
            time.sleep(1)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 12)
        f = io.StringIO()
        with redirect_stdout(f):
            time.sleep(2)
        output = f.getvalue()
        self.assertEqual(output.count("\n"), 20)
        task.stop()


if __name__ == '__main__':
    unittest.main()




import unittest
import os
import io
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from pelops.myconfigtools import read_config, validate_config
from pelops.mylogger import create_logger
from pelops.mymqttclient import MyMQTTClient
from lysidike.services.servicefactory import ServiceFactory
from lysidike.services.emailservice import EmailService
from lysidike.services.stdoutservice import StdOutService


class TestServiceFactory(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')) +
                                 "/tests_unit/config.yaml")
        cls.logger = create_logger(cls.config["logger"], "TestServiceFactory")
        cls.logger.info("start")
        cls.mqtt_client = MyMQTTClient(cls.config["mqtt"], cls.logger)
        cls.mqtt_client.connect()

    @classmethod
    def tearDownClass(cls):
        cls.mqtt_client.disconnect()
        cls.logger.info("end")

    def setUp(self):
        self.logger.info("-----------------------------------------------------------------------")

    def tearDown(self):
        self.logger.info(".......................................................................")

    def test_00create_stdout(self):
        config = self.config["publish-gateway"]["services"]["stdout"]
        service = ServiceFactory.get_service("stdout", config, self.mqtt_client, self.logger)
        self.assertIsNotNone(service)
        self.assertEqual(type(service), StdOutService)

    def test_01create_email(self):
        config = self.config["publish-gateway"]["services"]["email"]
        service = ServiceFactory.get_service("email", config, self.mqtt_client, self.logger)
        self.assertIsNotNone(service)
        self.assertEqual(type(service), EmailService)

    def test_10create_all(self):
        config = self.config["publish-gateway"]["services"]
        services = ServiceFactory.get_services(config, self.mqtt_client, self.logger)
        self.assertIsNotNone(services)
        self.assertEqual(len(services), 2)
        self.assertEqual(type(services["email"]), EmailService)
        self.assertEqual(type(services["stdout"]), StdOutService)


if __name__ == '__main__':
    unittest.main()




from setuptools import setup, find_packages
import os
import lysidike
import lysidike.publishgateway


def extract_path(fname):
    return os.path.join(os.path.dirname(__file__), fname)


def read(fname):
    return open(extract_path(fname)).read()


# convert README.md into README.rst - *.md is needed for gitlab; *.rst is needed for pypi
if os.path.isfile(extract_path('README.md')):
    try:
        from pypandoc import convert
        readme_rst = convert(extract_path('README.md'), 'rst')
        with open(extract_path('README.rst'), 'w') as out:
            out.write(readme_rst + '\n')
    except ModuleNotFoundError as e:
        print("Module pypandoc could not be imported - cannot update/generate README.rst.", e)


# update config schema json.
lysidike.publishgateway.PublishGateway.dump_schema(extract_path("config_schema.json"))

setup(
    name='Lysidike',
    version=lysidike.version,
    packages=find_packages(),
    license='MIT license',
    long_description=read('README.rst'),
    description="Lysidike publishes incoming mqtt messages to various internet services like email.",
    url='https://gitlab.com/pelops/lysidike/',
    author='Tobias Gawron-Deutsch',
    author_email='tobias@strix.at',
    keywords='mqtt device driver rpi raspberry pi',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        "Topic :: Home Automation",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.5',
    install_requires=[
        "pelops>=0.2.4",
    ],
    test_suite="tests_unit",
    entry_points={
        'console_scripts': [
            'lysidike = lysidike.publishgateway:standalone',
        ]
    },

)



def get_schema():
    schema = {
        "description": "a task consists of a target service, rules for when to send, and what to send",
        "type": "object",
        "properties": {
            "name": {
                "description": "name for the task",
                "type": "string"
            },
            "service": {
                "description": "one of the configured services e.g.'email'",
                "type": "string",
                "enum": ["email", "stdout"]
            },
            "subject": {
                "description": "subject for message",
                "type": "string"
            },
            "every-nth-message": {
                "description": "collect this amount of messages before publishing them. 0 for never",
                "type": "integer",
                "minimum": 0
            },
            "every-nth-second": {
                "description": "wait this time before publishing all messages that are waiting. 0 for never",
                "type": "number",
                "minimum": 0
            },
            "topics-sub": {
                "description": "use incoming messages from these topics",
                "type": "array",
                "items": {
                    "description": "mqtt topic",
                    "type": "string"
                },
                "additionalItems": False
            }
        },
        "required": ["name", "service", "subject", "every-nth-message", "every-nth-second", "topics-sub"],
        "additionalItems": False
    }

    return schema
